package com.ec.resteasy.providers.exceptionmapper;

import com.brilliance.isc.exception.RestBizException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;
import java.io.UnsupportedEncodingException;
import java.util.Base64;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * RESTEasy全局异常封装类,返回json格式异常信息;
 * 服务器端异常默认500
 *
 */
@Provider
@Component
public class RestExceptionMapper implements ExceptionMapper<Exception> {

	private Logger logger = LoggerFactory.getLogger(this.getClass());

	private static final int NOT_ACCEPTABLE_CODE = 406;

	private static final int NOT_ALLOWED_CODE = 405;

	private static final int NOT_FOUND_CODE = 404;

	private static final int NOT_AUTHORIZED_CODE = 401;

	private static final int BAD_REQUEST_CODE = 400;

	private static final int SERVER_ERROR_CODE = 500;

	private static final int BUFFER_SIZE = 1024;
	
	private static final String EXCEPTION_CALLBACK_SERIVCE_ID = "restExceptionCallbackSerivce";


	private int responseStatus = SERVER_ERROR_CODE;
	private String responseMessage = null;
	@Context
	HttpServletRequest request;

	@Context
	HttpServletResponse response;


	boolean first = true;
	
	public RestExceptionMapper(){
		logger.info("--RestExceptionMapper()--");
	}
	//原构造方法加载改为第一次发生异常是初始化 by yuyong 20220218
	public void init() {
		if (first) {
//			Configuration configuration = ConfigurationManager.getInstance().getConfiguration("ec");
//			String status = configuration.getValue("http.response.error.status");
//			if (StringUtils.isNotBlank(status)) {
//				try {
//					responseStatus = Integer.valueOf(status);
//				} catch (NumberFormatException e) {
//					logger.error(e);
//				}
//			}
//			responseMessage = configuration.getValue("http.response.error.message");
//			if (StringUtils.isBlank(responseMessage)) {
//				responseMessage = "访问出错，请联系管理员!";
//			}
			first = false;
			responseStatus = 200;
			responseMessage = "访问出错，请联系管理员!";
		}
	}
	
	/**
	 * 将异常信息封装为JSON格式
	 */
	@Override
	public Response toResponse(Exception e) {
		
		init();
		logger.error("发生业务异常{}",e);
		if (e instanceof BadRequestException) { // 400
			return Response.status(BAD_REQUEST_CODE).type(MediaType.APPLICATION_JSON).build();

		} else if (e instanceof NotAuthorizedException) { // 401
			return Response.status(NOT_AUTHORIZED_CODE).type(MediaType.APPLICATION_JSON).build();

		} else if (e instanceof NotFoundException) { // 404
			return Response.status(NOT_FOUND_CODE).type(MediaType.APPLICATION_JSON).build();

		} else if (e instanceof NotAllowedException) { // 405
			return Response.status(NOT_ALLOWED_CODE).type(MediaType.APPLICATION_JSON).build();

		} else if (e instanceof NotAcceptableException) { // 406
			return Response.status(NOT_ACCEPTABLE_CODE).type(MediaType.APPLICATION_JSON).build();

		}
//		else if (e instanceof DefaultOptionsMethodException){//跨域请求时，需要允许options请求
//			return ((DefaultOptionsMethodException) e).getResponse();
//		}
		else {
			return handle(e);
		}
	}
	
	/**
	 * 提供系统异常处理扩展接口 add by yxch
	 * @param e
	 * @return
	 */
	public Response handle(Exception e){
//		Map<String, ISystemExceptionHandler> systemExceptionHandlerMap = SpringWebApp.appContext.getBeansOfType(ISystemExceptionHandler.class);
//		if(!ObjectUtils.isEmpty(systemExceptionHandlerMap)){
//			try {
//				for (Entry<String, ISystemExceptionHandler> entry : systemExceptionHandlerMap.entrySet()) {
//					ISystemExceptionHandler systemExceptionHandler = entry.getValue();
//					if(logger.isDebugEnabled()){
//						logger.debug("调用系统级异常处理类：【"+systemExceptionHandler+"】");
//					}
//					return (Response)systemExceptionHandler.handle(e);
//				}
//			} catch (Exception ee) {
//				logger.error("调用系统级异常处理类异常",ee);
//				if(logger.isDebugEnabled()){
//					logger.debug("进行默认系统级异常处理");
//				}
//				return coreSystemExceptionHandler(ee);
//			}
//		}
		return coreSystemExceptionHandler(e);
	}
	
	public Response coreSystemExceptionHandler(Exception e){

		//新增异常处理接口,返回自定义异常信息 by yuyong 20220411
//		IRestExceptionCallbackSerivce callbackService =
//				(IRestExceptionCallbackSerivce)SpringWebApp.instance.getBean(EXCEPTION_CALLBACK_SERIVCE_ID);
//		if (callbackService != null) {
//			return callbackService.handlerException(e);
//		}
		//-----------end
		if(checkEdspFlag()) {
			return handlerEdspHeader(buildErrorMsg(e));
		}
		return Response.status(responseStatus).entity(buildErrorMsg(e)).header("nonEncrypt4Response", "true").type(MediaType.APPLICATION_JSON).build();
	}

	//添加了响应头 errorcode,errorMsg
	public Response handlerEdspHeader(RestErrorMsg errmsg) {
		String edspbase64="";
		try {
			edspbase64 = Base64.getEncoder().encodeToString(errmsg.getErrorMsg().toString().getBytes("utf-8"));
		} catch (UnsupportedEncodingException e1) {
			logger.error("base64加密异常!",e1);
		}
		if(edspbase64.length()>BUFFER_SIZE){
			edspbase64 = "";
		}
		return Response.status(responseStatus).entity(errmsg).header("nonEncrypt4Response", "true")
				.header("errorcode", errmsg.getErrorCode()).header("errorMsg",edspbase64)
				.type(MediaType.APPLICATION_JSON).build();
	}

	public boolean checkEdspFlag() {
//		Configuration configuration = ConfigurationManager.getInstance().getConfiguration("edsp");
//		boolean edsp = configuration != null && Boolean.valueOf(configuration.getValue("edspFlag"));
//		return edsp;
		return false;
	}

	/**
	 * 拼返回的JSON串并记录日志
	 * 
	 * @param e
	 * @return
	 */
	private RestErrorMsg buildErrorMsg(Exception e) {
		String errorCode = null;
		String errorMsg = null;


		if (e instanceof RestBizException) {
			errorCode = ((RestBizException) e).getCode();
			errorMsg = ((RestBizException) e).getMessage();
			if (errorMsg == null) {
				logger.error("系统处理异常!", e);
				errorMsg = responseMessage;
			}
		}
		else {
			// 未预期的异常打印日志
			logger.error("系统处理异常!", e);
			//尝试返回更明确的信息 by yuyong 20180821
			errorCode = "ERR9999";
			errorMsg = e.getMessage();
			if (errorMsg == null && e.getCause() != null) {
				errorMsg = e.getCause().getMessage();
			}
			if (errorMsg == null) {
				errorMsg = responseMessage;
			}
		}
		String traceid=response.getHeader("trace_id");
		logger.info("traceid is{}",traceid);
		errorMsg= String.format("%s|追踪流水号:[%s]",errorMsg,traceid);

//		errorMsg = handlerMessage(errorMsg);
		return new RestErrorMsg(errorCode, errorMsg);
	}

	private String handlerMessage(String errorMsg) {
		String regex = "([a-z]+\\.){2,}";
		Pattern p = Pattern.compile(regex);
		Matcher m = p.matcher(errorMsg);
		boolean isMatch = m.find();
		if (isMatch) {
			//qc114080 20201125
			 int start = errorMsg.indexOf("RestBizException:");
			  if (start != -1) {
				  errorMsg = errorMsg.substring(start + 17);
			   int end = errorMsg.indexOf("[");
			   if (end != -1) {
				   errorMsg = errorMsg.substring(0, end);
			   }
			  }else{
				  errorMsg = responseMessage;
			  }
		}
		//主要用于url异常时,不展示脚本标签,可能后续还需要添加其他的 by yuyong 20200513
		errorMsg = errorMsg.replaceAll("script", "");
		// 异常信息去掉ip显示 by jfy 20220609
		int index = errorMsg.indexOf("http://");
		if(index != -1){
			String errMsg = errorMsg;
			errMsg = errMsg.substring(index+7,errorMsg.length());
			String[] split = errMsg.split("/");
			if(split != null && split.length > 1){
				errMsg = "http://" + split[0];
			}
			errorMsg = errorMsg.replaceAll(errMsg, "");
		}
		return errorMsg;
	}

	private class RestErrorMsg {
		private String errorCode;
		private String errorMsg;
		public RestErrorMsg(String errorCode, String errorMsg) {
			this.errorCode = errorCode;
			this.errorMsg = errorMsg;
		}

		public String getErrorCode() {
			return errorCode;
		}

		public void setErrorCode(String errorCode) {
			this.errorCode = errorCode;
		}

		public String getErrorMsg() {
			return errorMsg;
		}

		public void setErrorMsg(String errorMsg) {
			this.errorMsg = errorMsg;
		}
	}
}
